// src/parser/Parser.js
const FunctionManager = require('./FunctionManager'); // To access defined functions

class Parser {
    constructor(botInstance) {
        this.bot = botInstance; // Access to client, config, etc.
        this.functions = new FunctionManager(botInstance, this); // Pass bot and parser
    }

    /**
     * The main entry point to parse and execute a Relay script.
     * @param {string} script - The raw string containing $functions from an .rl file.
     * @param {object} context - The execution context (e.g., message, interaction, author, channel, guild).
     * @returns {Promise<string | null>} The final result of the script execution (often the result of the last function), or null if execution was stopped or had no return value.
     */
    async parse(script, context) {
        if (!script || typeof script !== 'string') return null;

        // Initialize execution state for this run
        context.variables = context.variables || {}; // Simple key-value store for $let/$get
        context.stopExecution = false; // Flag to stop processing if $stop is called

        try {
            // First, substitute simple context variables (e.g., $authorID -> actual ID)
            // This makes them available directly within the script string before function evaluation
            // Careful with regex if values contain special characters!
            const substitutedScript = this.substituteContextVariables(script, context);

            // Now evaluate the script with functions
            const result = await this.evaluate(substitutedScript, context);

            // Check stop flag one last time
             if (context.stopExecution) return null;

            return result; // Return the final evaluated string
        } catch (error) {
            console.error("[Relay.js Parser Error] Error during script execution:", error);
            // Avoid logging full script/context in production if sensitive
            // console.error("[Relay.js Parser Error] Script:", script.substring(0, 200) + (script.length > 200 ? '...' : ''));
            // console.error("[Relay.js Parser Error] Context keys:", Object.keys(context));

            // Attempt to send an error message to Discord
            await this.sendErrorReply(error, context);
            return null; // Indicate error by returning null
        }
    }

     /**
     * Substitutes simple context variables like $authorID directly into the script string.
     * @param {string} script - The raw script.
     * @param {object} context - The execution context.
     * @returns {string} Script with variables substituted.
     * @private
     */
     substituteContextVariables(script, context) {
         // Use a regex to find $variableName patterns that are NOT followed by [
         // This avoids replacing function calls like $sendMessage[...]
         return script.replace(/\$([\w-]+)(?!\[)/g, (match, varName) => {
             const value = this.functions._getContextVariable(varName, context); // Use the getter from FunctionManager
             if (value !== undefined) {
                 // Basic escaping: if the value might interfere with parsing (e.g., contains $ or []), escape it.
                 // This is a simplistic approach and might need refinement.
                 return String(value); //.replace(/([$[\]])/g, '\\$1'); // Basic escaping - might be too broad
             }
             // If it's not a known context variable, check context.variables set by $let
              const letValue = context.variables?.[varName.toLowerCase()];
              if (letValue !== undefined) {
                  return String(letValue);
              }
             // If not found, leave the original text (e.g., $unknown)
             return match;
         });
     }

    /**
     * Evaluates a piece of script, handling functions and plain text.
     * This is the core recursive evaluation logic.
     * @param {string} text - The script text to evaluate (potentially with context vars substituted).
     * @param {object} context - The current execution context.
     * @returns {Promise<string | any>} The evaluated result. Can be non-string if a function returns it.
     * @private
     */
    async evaluate(text, context) {
        if (context.stopExecution) return null;

        let result = '';
        let currentPos = 0;

        while (currentPos < text.length) {
            if (context.stopExecution) break;

            // Find the next function call $funcName[
            let funcStartIndex = -1;
            let funcName = '';
            let argsStartIndex = -1;

            // Search for the next '$' that isn't escaped and is followed by name and '['
            let searchPos = currentPos;
            while (searchPos < text.length) {
                 const dollarIndex = text.indexOf('$', searchPos);
                 if (dollarIndex === -1) break; // No more '$' found

                 // Check if escaped (simple check for '\$')
                 if (dollarIndex > 0 && text[dollarIndex - 1] === '\\') {
                     searchPos = dollarIndex + 1;
                     continue;
                 }

                 // Check for valid function name and opening bracket
                 const match = text.substring(dollarIndex).match(/^\$([\w-]+)\[/);
                 if (match) {
                    funcStartIndex = dollarIndex;
                    funcName = match[1];
                    argsStartIndex = funcStartIndex + match[0].length; // Position after '['
                    break; // Found a valid function start
                 } else {
                     // It's a '$' but not a valid function start, move past it
                     searchPos = dollarIndex + 1;
                 }
            }


            if (funcStartIndex !== -1) {
                // Append plain text before the function call
                if (funcStartIndex > currentPos) {
                    result += this.unescapeText(text.substring(currentPos, funcStartIndex));
                }

                // Find the matching closing bracket for this function call
                const argsEndIndex = this.findMatchingBracket(text, argsStartIndex);
                if (argsEndIndex === -1) {
                    throw new Error(`Parser Error: Unmatched bracket '[' for function "$${funcName}" starting near position ${funcStartIndex}.`);
                }

                const argsString = text.substring(argsStartIndex, argsEndIndex);
                const args = await this.parseArguments(argsString, context);

                if (context.stopExecution) break; // Stop if argument parsing triggered $stop

                // Execute the function
                const funcResult = await this.functions.execute(funcName, args, context);

                // Append result (convert to string unless specifically handled otherwise)
                if (funcResult !== undefined && funcResult !== null) {
                    result += String(funcResult); // Ensure result is stringified for concatenation
                }

                currentPos = argsEndIndex + 1; // Move past the closing bracket ']'
            } else {
                // No more functions found, append the rest of the plain text
                if (currentPos < text.length) {
                     result += this.unescapeText(text.substring(currentPos));
                }
                break; // End of processing
            }
        }

        // Final unescaping of the entire result might be needed if functions returned text with escaped chars
         // return this.unescapeText(result);
         return result; // Return potentially mixed-type result if needed, otherwise string
    }

    /**
     * Finds the matching closing bracket ']' for a function call, respecting nesting.
     * @param {string} text - The script text.
     * @param {number} startIndex - The position immediately after the opening '['.
     * @returns {number} The index of the matching ']', or -1 if not found.
     * @private
     */
    findMatchingBracket(text, startIndex) {
        let depth = 1;
        for (let i = startIndex; i < text.length; i++) {
             // Basic check for escaped characters - might need refinement
            if (text[i-1] === '\\') continue;

            if (text[i] === '[' && text[i-1] === '$') { // Look for nested $func[ more accurately
                depth++;
            } else if (text[i] === ']') {
                depth--;
                if (depth === 0) {
                    return i; // Found the matching bracket
                }
            }
        }
        return -1; // Not found
    }

    /**
     * Parses the argument string of a function, splitting by ';' and evaluating each part.
     * Respects nested functions and escaped semicolons.
     * @param {string} argsString - The raw string between '[' and ']'.
     * @param {object} context - The current execution context.
     * @returns {Promise<Array<any>>} An array of evaluated arguments.
     * @private
     */
    async parseArguments(argsString, context) {
        const args = [];
        let depth = 0;
        let currentArgStart = 0;

        for (let i = 0; i < argsString.length; i++) {
             if (context.stopExecution) break; // Check stop flag

             // Basic check for escaped characters
             if (argsString[i-1] === '\\') continue;

             if (argsString[i] === '[' && argsString[i-1] === '$') {
                depth++;
            } else if (argsString[i] === ']') {
                 if (depth > 0) depth--; // Only decrease depth if it's positive
            } else if (argsString[i] === ';' && depth === 0) {
                // Found a separator at the top level
                const argToEvaluate = argsString.substring(currentArgStart, i); // Don't trim yet
                args.push(await this.evaluate(argToEvaluate, context)); // Evaluate the segment
                currentArgStart = i + 1; // Move start position past the semicolon
            }
        }

        // Evaluate the last argument (or the only argument if no semicolons)
        if (!context.stopExecution && currentArgStart <= argsString.length) {
             const lastArgToEvaluate = argsString.substring(currentArgStart);
             args.push(await this.evaluate(lastArgToEvaluate, context));
        }

        // If the original string was empty, ensure one empty argument is returned if needed by functions
         if (argsString.length === 0 && args.length === 0) {
             args.push('');
         }

        return args;
    }

    /**
     * Removes escape characters (\) from text segments.
     * @param {string} text
     * @returns {string}
     * @private
     */
     unescapeText(text) {
         // Replace escaped characters like \$ \[ \] \; with their literal counterparts
         return text.replace(/\\([$[\];\\])/g, '$1');
     }

     /**
     * Sends an error reply to Discord if possible.
     * @param {Error} error - The error object.
     * @param {object} context - The execution context.
     * @private
     */
     async sendErrorReply(error, context) {
         const errorMessage = `❌ Error: ${error.message || 'An unknown error occurred.'}`;
         try {
             if (context.interaction) {
                 if (context.interaction.replied || context.interaction.deferred) {
                     await context.interaction.followUp({ content: errorMessage, ephemeral: true });
                 } else {
                     await context.interaction.reply({ content: errorMessage, ephemeral: true });
                 }
             } else if (context.message?.channel && !error.message?.includes("reply")) {
                 // Avoid reply loops if the error originated from a reply attempt
                 await context.message.channel.send(errorMessage);
             }
         } catch (discordError) {
             console.error("[Relay.js] Failed to send error message to Discord:", discordError);
         }
     }
}

module.exports = Parser;